/*
    Copyright  1995-2001, The AROS Development Team. All rights reserved.
    $Id: asl_init.c,v 1.2 2007/10/28 11:09:21 piru Exp $

    Desc: Asl initialization code.
    Lang: English.
*/


#include <stddef.h>
#include <exec/libraries.h>
#include <exec/lists.h>
#include <exec/resident.h>
#include <dos/dosextens.h>
#include <aros/libcall.h>
#include <aros/asmcall.h>
#include <intuition/screens.h>
#include <graphics/modeid.h>

#include <proto/exec.h>
#include <proto/intuition.h>

#include "initstruct.h"
#include "asl_intern.h"
#include "libdefs.h"
#ifdef __MORPHOS__
#ifdef VERSION_STRING
#undef VERSION_STRING
#endif
#ifdef VERSION_NUMBER
#undef VERSION_NUMBER
#endif
#ifdef REVISION_NUMBER
#undef REVISION_NUMBER
#endif
#include "asl_rev.h"
#define VERSION_STRING VERSTAG
#define VERSION_NUMBER VERSION
#define REVISION_NUMBER REVISION
#endif

#define CATCOMP_NUMBERS
#include "asl_strings.h"

#define INIT AROS_SLIB_ENTRY(init, Asl)

#define DEBUG 0
#include <aros/debug.h>

/*****************************************************************************************/

#ifdef __MORPHOS__
    unsigned long __abox__ = 1;
    unsigned long __amigappc__ = 1;
#endif

struct inittable;
extern const char name[];
extern const char version[];
extern const APTR inittabl[4];
extern void *const LIBFUNCTABLE[];
extern const struct inittable datatable;
extern struct AslBase_intern *INIT();
extern struct AslBase_intern *AROS_SLIB_ENTRY(open,Asl)();
extern BPTR AROS_SLIB_ENTRY(close,Asl)();
extern BPTR AROS_SLIB_ENTRY(expunge,Asl)();
extern int AROS_SLIB_ENTRY(null,Asl)();
extern const char LIBEND;

/* FIXME: egcs 1.1b and possibly other incarnations of gcc have
 * two nasty problems with entry() that prevents using the
 * C version of this function on AROS 68k native.
 *
 * First of all, if inlining is active (-O3), the optimizer will decide
 * that entry() is simple enough to be inlined, and it doesn't generate
 * its code until all other functions have been compiled. Delaying asm
 * output for a global (non static) function is probably silly because
 * the optimizer can't eliminate its stand alone istance anyway.
 *
 * The second problem is that even without inlining, the code generator
 * adds a nop instruction immediately after rts. This is probably done
 * to help the 68040/60 pipelines, but it adds two more bytes before the
 * library resident tag, which causes all kinds of problems on native
 * AmigaOS.
 *
 * The workaround is to embed the required assembler instructions
 * (moveq #-1,d0 ; rts) in a constant variable.
 */
#if (defined(__mc68000__) && (AROS_FLAVOUR & AROS_FLAVOUR_NATIVE))
const LONG entry = 0x70FF4E75;
#else
int entry(void)
{
    /* If the library was executed by accident return error code. */
    return -1;
}
#endif

static const struct Resident resident=
{
    RTC_MATCHWORD,
    (struct Resident *)&resident,
    (APTR)&LIBEND,
#ifdef __MORPHOS__
    RTF_PPC | RTF_EXTENDED | RTF_AUTOINIT,
#else
    RTF_AUTOINIT,
#endif
    VERSION_NUMBER,
    NT_LIBRARY,
    0,	/* priority */
    (char *)name,
    (char *)&version[6],
    (ULONG *)inittabl,
#ifdef __MORPHOS__
    REVISION_NUMBER,	/* Revision */
    NULL /* Tags */
#endif
};

const char name[]=NAME_STRING;

const char version[]=VERSION_STRING;

const APTR inittabl[4]=
{
    (APTR)sizeof(struct AslBase_intern),
    (APTR)LIBFUNCTABLE,
#ifdef __MORPHOS__
    NULL,
#else
    (APTR)&datatable,
#endif
    &INIT
};

#ifndef __MORPHOS__
struct inittable
{
    S_CPYO(1,1,B);
    S_CPYO(2,1,L);
    S_CPYO(3,1,B);
    S_CPYO(4,1,W);
    S_CPYO(5,1,W);
    S_CPYO(6,1,L);
    S_END (LIBEND);
};

#define O(n) offsetof(struct AslBase_intern,n)

const struct inittable datatable=
{
    { { I_CPYO(1,B,O(library.lib_Node.ln_Type)), { NT_LIBRARY } } },
    { { I_CPYO(1,L,O(library.lib_Node.ln_Name)), { (IPTR)name } } },
    { { I_CPYO(1,B,O(library.lib_Flags       )), { LIBF_SUMUSED|LIBF_CHANGED } } },
    { { I_CPYO(1,W,O(library.lib_Version     )), { VERSION_NUMBER } } },
    { { I_CPYO(1,W,O(library.lib_Revision    )), { REVISION_NUMBER } } },
    { { I_CPYO(1,L,O(library.lib_IdString    )), { (IPTR)&version[6] } } },
  I_END ()
};
#endif

/* #undef O
#undef SysBase */

/*****************************************************************************************/

#define DEF_WIDTH	400
#define DEF_HEIGHT	500

/* Requester type specific default data */
const struct IntFileReq def_filereq =
{
    {
	ASL_FileRequest,
	NULL,			/* Window		*/
	NULL,			/* Screen		*/
	NULL,			/* PubScreenName	*/
	NULL,			/* IntuiMsgFunc 	*/
	NULL,			/* TextAttr		*/
	NULL,			/* Locale		*/
	NULL,			/* Catalog		*/
	NULL,			/* MemPool		*/
	2048,			/* MemPoolPuddle	*/
	2048,			/* MemPoolThresh	*/
	MSG_FILEREQ_TITLE,   	/* TitleID  	    	*/
	NULL,		    	/* TitleText		*/
	NULL,			/* PositiveText		*/
	NULL,			/* NegativeText		*/
	-1, -1,	 		/* --> center on screen */
	DEF_WIDTH, DEF_HEIGHT	/* Width/Height		*/
    },

    "",				/* File 	 	*/
    "",     			/* Drawer        	*/
    "#?",       		/* Pattern       	*/
    NULL,			/* AcceptPattern 	*/ /* def. = "#?", but must be ParsePatternNoCase'ed */
    NULL,			/* RejectPattern 	*/ /* def. = "~(#?)", but must be ParsePatternNoCase'ed */
    0,				/* Flags1	 	*/
    FRF_REJECTICONS,		/* Flags2	 	*/
    NULL,			/* FilterFunc	 	*/
    NULL,			/* HookFunc	 	*/
    NULL,			/* GetSortBy	 	*/
    NULL,			/* GetSortOrder  	*/
    NULL,			/* GetSortDrawers	*/
    ASLFRSORTBY_Name,   	/* SortBy        	*/
    ASLFRSORTORDER_Ascend,	/* SortOrder     	*/
    ASLFRSORTDRAWERS_First,	/* SortDrawers   	*/
    FALSE			/* InitialShowVolumes 	*/
};

/*****************************************************************************************/

const struct IntSMReq def_smreq =
{
    {
	ASL_ScreenModeRequest,
	NULL,				/* Window		*/
	NULL,				/* Screen		*/
	NULL,				/* PubScreenName	*/
	NULL,				/* IntuiMsgFunc 	*/
	NULL,				/* TextAttr		*/
	NULL,				/* Locale		*/
	NULL,				/* Catalog		*/
	NULL,				/* MemPool		*/
	2048,				/* MemPoolPuddle	*/
	2048,				/* MemPoolThresh	*/
	MSG_MODEREQ_TITLE,   	    	/* TitleID  	    	*/
	NULL,		    	    	/* TitleText		*/
	NULL,				/* PositiveText		*/
	NULL,			    	/* NegativeText		*/
	-1, -1,	 			/* --> center on screen */
	DEF_WIDTH, DEF_HEIGHT		/* Width/Height		*/
    },

    NULL,				/* CustomSMList 	*/
    NULL,				/* FilterFunc 		*/
    0,					/* Flags 		*/
    LORES_KEY,				/* DisplayID 		*/
    640,				/* DisplayWidth 	*/
    200,				/* DisplayHeight 	*/
    640,				/* BitMapWidth 		*/
    200,				/* BitMapHeight 	*/
    2,					/* DisplayDepth 	*/
    OSCAN_TEXT,				/* OverscanType 	*/
    TRUE,				/* AutoScroll 		*/
    DIPF_IS_WB,				/* PropertyFlags 	*/
    DIPF_IS_WB,				/* PropertyMask 	*/
    1,					/* MinDepth 		*/
    /*24*/ 32,				/* MaxDepth 		*/
    16,					/* MinWidth 		*/
    16384,				/* MaxWidth 		*/
    16,					/* MinHeight 		*/
    16384,				/* MaxHeight 		*/
    20,					/* InfoLeftEdge 	*/
    20,					/* InfoTopEdge 		*/
    FALSE				/* InfoOpened 		*/
};

/*****************************************************************************************/

const struct IntFontReq def_fontreq =
{
    {
	ASL_FontRequest,
	NULL,				/* Window		*/
	NULL,				/* Screen		*/
	NULL,				/* PubScreenName	*/
	NULL,				/* IntuiMsgFunc 	*/
	NULL,				/* TextAttr		*/
	NULL,				/* Locale		*/
	NULL,				/* Catalog		*/
	NULL,				/* MemPool		*/
	2048,				/* MemPoolPuddle	*/
	2048,				/* MemPoolThresh	*/
	MSG_FONTREQ_TITLE,   	    	/* TitleID  	    	*/
	NULL,			    	/* TitleText		*/
	NULL,				/* PositiveText		*/
	NULL,			    	/* NegativeText		*/
	-1, -1,				/* --> center on screen */
	DEF_WIDTH, DEF_HEIGHT		/* Width/Height		*/
    },
    {"topaz", 8, FS_NORMAL,FPF_ROMFONT},/* Default textattr 	*/
    1,				    	/* FrontPen		*/
    0,			    	    	/* BackPen		*/
    JAM1,				/* DrawMode		*/
    0,					/* Flags		*/

    5,					/* Minheight		*/
    24,				    	/* MaxHeight		*/
    NULL,			    	/* FilterFunc		*/
    NULL,				/* HookFunc		*/
    32, 				/* MaxFrontPen		*/
    32, 				/* MaxBackPen		*/

    NULL,				/* ModeList		*/
    NULL,				/* FrontPens		*/
    NULL,				/* BackPens		*/

};

/*****************************************************************************************/

VOID InitReqInfo(struct AslBase_intern *);

/*****************************************************************************************/

/* I have to make IntuitionBase global to make use of NewObject() */
#ifdef GLOBAL_INTUIBASE
struct IntuitionBase * IntuitionBase;
#endif

/* snprintf() need global SysBase & DOSBase */
#ifdef GLOBAL_SYSBASE
struct ExecBase *SysBase;
#endif

#ifdef GLOBAL_DOSBASE
struct DosLibrary *DOSBase;
#endif

/*****************************************************************************************/

#ifdef __MORPHOS__
struct AslBase_intern *LIB_init(struct AslBase_intern *LIBBASE, BPTR segList, struct ExecBase *sysBase)
{
#else
AROS_UFH3(struct AslBase_intern *, AROS_SLIB_ENTRY(init,BASENAME),
    AROS_UFHA(struct AslBase_intern *, LIBBASE, D0),
    AROS_UFHA(BPTR,               segList,   A0),
    AROS_UFHA(struct ExecBase *, sysBase, A6)
)
#endif
{
    AROS_USERFUNC_INIT

    /* This function is single-threaded by exec by calling Forbid. */

    SysBase=sysBase;

    D(bug("Inside initfunc\n"));

    LIBBASE->seglist=segList;

    /* Asl specific initialization stuff */
    NEWLIST(&LIBBASE->ReqList);

    InitSemaphore(&LIBBASE->ReqListSem);

    InitReqInfo(LIBBASE);

#ifdef __MORPHOS__
    LIBBASE->library.lib_Revision = REVISION_NUMBER;
#endif

    /* You would return NULL here if the init failed. */
    return LIBBASE;
    AROS_USERFUNC_EXIT
}

/*****************************************************************************************/

AROS_LH1(struct AslBase_intern *, open,
    AROS_LHA(ULONG, version, D0),
    struct AslBase_intern *, LIBBASE, 1, BASENAME)
{
    AROS_LIBFUNC_INIT
    /*
	This function is single-threaded by exec by calling Forbid.
	If you break the Forbid() another task may enter this function
	at the same time. Take care.
    */

    /*
	What the original comment writer above failed to note is that
	expunge while lib_Opencnt is 0 leads into flushing of the
	library code segment.

	This is why we need to bump the lib_OpenCnt *before* Forbid()
	is (possibly) broken. OpenLibrary will break forbid, if the
	library need to be loaded from the disk. - Piru
    */

    LIBBASE->library.lib_OpenCnt++;

    /* Keep compiler happy */
    (void) version;

    /* Is it the first OpenLibrary call? We only open these libraries
       once to speed up further library opens. - Piru
     */
    if (LIBBASE->library.lib_OpenCnt == 1)
    {

	D(bug("Inside openfunc\n"));

	if (!DOSBase)
	    DOSBase = (struct DosLibrary *)OpenLibrary("dos.library", 37);
	if (!DOSBase)
	{
	    LIBBASE->library.lib_OpenCnt--;
	    return NULL;
	}

#if defined(__MORPHOS__) && USE_DOS64
	LIBBASE->UseDOS64 = LIB_MINVER(&DOSBase->dl_lib, 51, 28);
#endif

	if (!GfxBase)
	    GfxBase = (GraphicsBase *)OpenLibrary("graphics.library", 37);
	if (!GfxBase)
	{
	    LIBBASE->library.lib_OpenCnt--;
	    return NULL;
	}

	if (!LayersBase)
	    LayersBase = OpenLibrary("layers.library", 37);
	if (!LayersBase)
	{
	    LIBBASE->library.lib_OpenCnt--;
	    return NULL;
	}

	if (!DiskfontBase)
	    DiskfontBase = OpenLibrary("diskfont.library", 37);
	if (!DiskfontBase)
	{
	    LIBBASE->library.lib_OpenCnt--;
	    return NULL;
	}

	if (!UtilityBase)
	    UtilityBase = OpenLibrary("utility.library", 37);
	if (!UtilityBase)
	{
	    LIBBASE->library.lib_OpenCnt--;
	    return NULL;
	}

	if (!GadToolsBase)
	    GadToolsBase = OpenLibrary("gadtools.library", 37);
	if (!GadToolsBase)
	{
	    LIBBASE->library.lib_OpenCnt--;
	    return NULL;
	}

	if (!IntuitionBase)
	    IntuitionBase = (IntuiBase *)OpenLibrary("intuition.library", 37);
	if (!IntuitionBase)
	{
	    LIBBASE->library.lib_OpenCnt--;
	    return NULL;
	}

	if (!LIBBASE->aslpropclass)
	    LIBBASE->aslpropclass = makeaslpropclass(LIBBASE);
	if (!LIBBASE->aslpropclass)
	{
	    LIBBASE->library.lib_OpenCnt--;
	    return NULL;
	}

	if (!LIBBASE->aslarrowclass)
	    LIBBASE->aslarrowclass = makeaslarrowclass(LIBBASE);
	if (!LIBBASE->aslarrowclass)
	{
	    LIBBASE->library.lib_OpenCnt--;
	    return NULL;
	}

	if (!LIBBASE->asllistviewclass)
	    LIBBASE->asllistviewclass = makeasllistviewclass(LIBBASE);
	if (!LIBBASE->asllistviewclass)
	{
	    LIBBASE->library.lib_OpenCnt--;
	    return NULL;
	}

	if (!LIBBASE->aslbuttonclass)
	    LIBBASE->aslbuttonclass = makeaslbuttonclass(LIBBASE);
	if (!LIBBASE->aslbuttonclass)
	{
	    LIBBASE->library.lib_OpenCnt--;
	    return NULL;
	}

	if (!LIBBASE->aslstringclass)
	    LIBBASE->aslstringclass = makeaslstringclass(LIBBASE);
	if (!LIBBASE->aslstringclass)
	{
	    LIBBASE->library.lib_OpenCnt--;
	    return NULL;
	}

	if (!LIBBASE->aslcycleclass)
	    LIBBASE->aslcycleclass = makeaslcycleclass(LIBBASE);
	if (!LIBBASE->aslcycleclass)
	{
	    LIBBASE->library.lib_OpenCnt--;
	    return NULL;
	}

	if (!LIBBASE->aslfontpreviewclass)
	    LIBBASE->aslfontpreviewclass = makeaslfontpreviewclass(LIBBASE);
	if (!LIBBASE->aslfontpreviewclass)
	{
	    LIBBASE->library.lib_OpenCnt--;
	    return NULL;
	}

	if (!LIBBASE->aslfontstyleclass)
	    LIBBASE->aslfontstyleclass = makeaslfontstyleclass(LIBBASE);
	if (!LIBBASE->aslfontstyleclass)
	{
	    LIBBASE->library.lib_OpenCnt--;
	    return NULL;
	}

	if (!LIBBASE->aslcolorpickerclass)
	    LIBBASE->aslcolorpickerclass = makeaslcolorpickerclass(LIBBASE);
	if (!LIBBASE->aslcolorpickerclass)
	{
	    LIBBASE->library.lib_OpenCnt--;
	    return NULL;
	}
    }

    /* We can live without cybergraphics.library so don't abort if opening fails */
    if (!CyberGfxBase)
	CyberGfxBase = OpenLibrary("cybergraphics.library", 40);

    /* We can live without locale.library so don't abort if opening fails */
    if (!LocaleBase)
	LocaleBase = OpenLibrary("locale.library", 38);

#if USE_SHARED_COOLIMAGES
    /* We can live without coolimages.library so don't abort if opening fails */
    if (!CoolImagesBase)
	CoolImagesBase = OpenLibrary("coolimages.library", 1);

#endif

    /* ------------------------- */

    /* I have one more opener. Already bumped above - Piru */
    //LIBBASE->library.lib_OpenCnt++;
    LIBBASE->library.lib_Flags &= ~LIBF_DELEXP;

    /* You would return NULL if the open failed. */
    return LIBBASE;
    AROS_LIBFUNC_EXIT
}

/*****************************************************************************************/

AROS_LH0(BPTR, close, struct AslBase_intern *, LIBBASE, 2, BASENAME)
{
    AROS_LIBFUNC_INIT
    /*
	This function is single-threaded by exec by calling Forbid.
	If you break the Forbid() another task may enter this function
	at the same time. Take care.
    */

    /* I have one fewer opener. */
    if (!--LIBBASE->library.lib_OpenCnt)
    {

	if (LIBBASE->aslpropclass)
	{
	    FreeClass(LIBBASE->aslpropclass);
	    LIBBASE->aslpropclass = NULL;
	}

	if (LIBBASE->aslarrowclass)
	{
	    FreeClass(LIBBASE->aslarrowclass);
	    LIBBASE->aslarrowclass = NULL;
	}

	if (LIBBASE->asllistviewclass)
	{
	    FreeClass(LIBBASE->asllistviewclass);
	    LIBBASE->asllistviewclass = NULL;
	}

	if (LIBBASE->aslbuttonclass)
	{
	    FreeClass(LIBBASE->aslbuttonclass);
	    LIBBASE->aslbuttonclass = NULL;
	}

	if (LIBBASE->aslstringclass)
	{
	    FreeClass(LIBBASE->aslstringclass);
	    LIBBASE->aslstringclass = NULL;
	}

	if (LIBBASE->aslcycleclass)
	{
	    FreeClass(LIBBASE->aslcycleclass);
	    LIBBASE->aslcycleclass = NULL;
	}

	if (LIBBASE->aslfontpreviewclass)
	{
	    FreeClass(LIBBASE->aslfontpreviewclass);
	    LIBBASE->aslfontpreviewclass = NULL;
	}

	if (LIBBASE->aslfontstyleclass)
	{
	    FreeClass(LIBBASE->aslfontstyleclass);
	    LIBBASE->aslfontstyleclass = NULL;
	}

	if (LIBBASE->aslcolorpickerclass)
	{
	    FreeClass(LIBBASE->aslcolorpickerclass);
	    LIBBASE->aslcolorpickerclass = NULL;
	}

    #if USE_SHARED_COOLIMAGES
	if (CoolImagesBase)
	{
	    CloseLibrary(CoolImagesBase);
	    CoolImagesBase = NULL;
	}
    #endif

	if (LocaleBase)
	{
	    CloseLibrary(LocaleBase);
	    LocaleBase = NULL;
	}

	if (CyberGfxBase)
	{
	    CloseLibrary(CyberGfxBase);
	    CyberGfxBase = NULL;
	}

	if (DiskfontBase)
	{
	    CloseLibrary(DiskfontBase);
	    DiskfontBase = NULL;
	}

	if (GadToolsBase)
	{
	    CloseLibrary(GadToolsBase);
	    GadToolsBase = NULL;
	}

	if (UtilityBase)
	{
	    CloseLibrary(UtilityBase);
	    UtilityBase = NULL;
	}

	if (LayersBase)
	{
	    CloseLibrary(LayersBase);
	    LayersBase = NULL;
	}

	if (GfxBase)
	{
	    CloseLibrary((struct Library *)GfxBase);
	    GfxBase = NULL;
	}

	if (DOSBase)
	{
	    CloseLibrary((struct Library *)DOSBase);
	    DOSBase = NULL;
	}

	if (IntuitionBase)
	{
	    CloseLibrary((struct Library *)IntuitionBase);
	    IntuitionBase = NULL;
	}

	/* Delayed expunge pending? */
	if (LIBBASE->library.lib_Flags & LIBF_DELEXP)
	{
	    /* Then expunge the library */
	    return expunge();
	}
    }

    return 0;

    AROS_LIBFUNC_EXIT
}

/*****************************************************************************************/

AROS_LH0(BPTR, expunge, struct AslBase_intern *, LIBBASE, 3, BASENAME)
{
    AROS_LIBFUNC_INIT

    BPTR ret;
    /*
	This function is single-threaded by exec by calling Forbid.
	Never break the Forbid() or strange things might happen.
    */

    /* Test for openers. */
    if (LIBBASE->library.lib_OpenCnt)
    {
	/* Set the delayed expunge flag and return. */
	LIBBASE->library.lib_Flags |= LIBF_DELEXP;
	return 0;
    }

    /* Get rid of the library. Remove it from the list. */
    Forbid();
    Remove(&LIBBASE->library.lib_Node);
    Permit();

    /* Get returncode here - FreeMem() will destroy the field. */
    ret = LIBBASE->seglist;

    /* Free the memory. */
    FreeMem((char *)LIBBASE-LIBBASE->library.lib_NegSize,
	LIBBASE->library.lib_NegSize+LIBBASE->library.lib_PosSize);

    return ret;
    AROS_LIBFUNC_EXIT
}

/*****************************************************************************************/

AROS_LH0I(int, null, struct AslBase_intern *, LIBBASE, 4, BASENAME)
{
    AROS_LIBFUNC_INIT

    return 0;

    AROS_LIBFUNC_EXIT
}

/*****************************************************************************************/

#include <string.h>
#include "filereqhooks.h"
#include "fontreqhooks.h"
#include "modereqhooks.h"

/*****************************************************************************************/

VOID InitReqInfo(struct AslBase_intern *AslBase)
{
    struct AslReqInfo *reqinfo;

    /* Set file requester info */

    reqinfo = &(ASLB(AslBase)->ReqInfo[ASL_FileRequest]);
    reqinfo->IntReqSize 	= sizeof (struct IntFileReq);
    reqinfo->ReqSize		= sizeof (struct FileRequester);
    reqinfo->DefaultReq 	= (struct IntFileReq *)&def_filereq;
    reqinfo->UserDataSize	= sizeof (struct FRUserData);

    bzero(&(reqinfo->ParseTagsHook), sizeof (struct Hook));
    bzero(&(reqinfo->GadgetryHook), sizeof (struct Hook));
    reqinfo->ParseTagsHook.h_Entry	= (void *)AROS_ASMSYMNAME(FRTagHook);
    reqinfo->GadgetryHook.h_Entry	= (void *)AROS_ASMSYMNAME(FRGadgetryHook);

    /* Set font requester info */

    reqinfo = &(ASLB(AslBase)->ReqInfo[ASL_FontRequest]);
    reqinfo->IntReqSize 	= sizeof (struct IntFontReq);
    reqinfo->ReqSize		= sizeof (struct FontRequester);
    reqinfo->DefaultReq 	= (struct IntFontReq *)&def_fontreq;
    reqinfo->UserDataSize	= sizeof (struct FOUserData);

    bzero(&(reqinfo->ParseTagsHook), sizeof (struct Hook));
    bzero(&(reqinfo->GadgetryHook), sizeof (struct Hook));
    reqinfo->ParseTagsHook.h_Entry	= (void *)AROS_ASMSYMNAME(FOTagHook);
    reqinfo->GadgetryHook.h_Entry	= (void *)AROS_ASMSYMNAME(FOGadgetryHook);

    /* Set screenmode requester info */

    reqinfo = &(ASLB(AslBase)->ReqInfo[ASL_ScreenModeRequest]);
    reqinfo->IntReqSize 	= sizeof (struct IntSMReq);
    reqinfo->ReqSize		= sizeof (struct ScreenModeRequester);
    reqinfo->DefaultReq 	= (struct IntSMReq *)&def_smreq;
    reqinfo->UserDataSize	= sizeof(struct SMUserData);

    bzero(&(reqinfo->ParseTagsHook), sizeof (struct Hook));
    bzero(&(reqinfo->GadgetryHook), sizeof (struct Hook));
    reqinfo->ParseTagsHook.h_Entry	= (void *)AROS_ASMSYMNAME(SMTagHook);
    reqinfo->GadgetryHook.h_Entry	= (void *)AROS_ASMSYMNAME(SMGadgetryHook);

    return;
}

/*****************************************************************************************/
